#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''*****************************************************************************
***  This file is part of HgWeb Manager.                                     ***
***                                                                          ***
***  Copyright (C) 2014 - 2015                                               ***
***  CodeGNU Solutions <Licensing _AT_ CodeGNU _DOT_ com>                    ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''


def is_hgWeb_user(expectedHgUser):
	""" Determines if the current running user is the expected HgWeb user.

	@param[in] str expectedHgUser	The expected user-name for the Mercurial
									web-manager.

	@return Gives true when the user is the expected HgWeb user, false
	else-wise.
	"""
	import getpass

	# FIXME: This isn't a secure check as it is based upon the environmental variable.
	if expectedHgUser == getpass.getuser():
		return True
	else:
		return False


def __extract_values(args):
	""" Pulls the creation arguments out of the command-line. The returned
	object is filled with the repository details when it exists and any of the
	detail values supplied on the command-line will replace the one found in
	the repository details.

	@param[in] args	The command-line argument processor containing the
					creation argument values.

	@return A managed repository object.
	"""
	import manrepo

	managed = None
	if args.rename:
		original = manrepo.Repository(args.storage)
		managed = manrepo.Repository(args.rename)
		managed.DisplayName = original.DisplayName
		managed.Description = original.Description
		managed.Contact = original.Contact
	else:
		managed = manrepo.Repository(args.storage)

	if args.name:
		managed.DisplayName = args.name

	if args.description:
		managed.Description = args.description

	if args.contact:
		managed.Contact = args.contact

	return managed

if __name__ == '__main__':
	import _app_info

	try:
		# Display license
		print "%s %s Copyright (C) %s CodeGNU Solutions" % \
			(_app_info.LONG_NAME, _app_info.VERSION, _app_info.YEARS)
		print "%s comes with ABSOLUTELY NO WARRANTY;" % (_app_info.SHORT_NAME)
		print 'This is free software, and you are welcome to redistribute it'
		print 'under certain conditions; see the LICENSE file for details,'
		print 'or the Free Software Foundation\'s AGPL.\n'

		from manager import settings
		import argparse
		import manager
		import sys

		if not is_hgWeb_user(settings.HgUser):
			print >>sys.stderr, 'Must execute as the Mercurial Web manager.'
			exit(1)

		parser = argparse.ArgumentParser(description=_app_info.DESCRIPTION)
		parser.add_argument('-l', '--list', action='store_true', help='Outputs a list of all the managed repositories.')

		plgInGrp = parser.add_argument_group('Manage Plug-Ins', 'Options that operate upon a managed repository\'s plug-ins.')
		plgInGrp.add_argument('-p', '--plugin', action='store', nargs=1, type=str, default=None, help='The name of a plug-in for a repository.')
		plgInGrp.add_argument('-e', '--enable', action='store_true', help='Enable a plug-in in a repository.')
		plgInGrp.add_argument('-f', '--script-path', action='store', nargs='?', type=str, default=None, help='The path to the plug-in script.')
		plgInGrp.add_argument('-x', '--disable', action='store_true', help='Disable a plug-in in a repository.')
		plgInGrp.add_argument('-s', '--setting', action='store', nargs=2, type=str, default=None, help='Sets a setting name/value pair for a plug-in.')

		manGrp = parser.add_argument_group('Manage Repositories', 'Options that operate upon managed repositories.')
		manGrp.add_argument('storage', action='store', nargs='?', help='The storage-name of the repository to perform an action upon.')
		manGrp.add_argument('-a', '--action', action='store', nargs=1, type=str, choices=['c', 'create', 'm', 'modify', 'd', 'delete', 'r', 'register'], help='Performs an action upon a managed repository.')
		manGrp.add_argument('-i', '--ignore', action='store', nargs='+', default=None, help='The groups to place in an ignore file.')
		manGrp.add_argument('-k', '--skip-common', action='store_true', help='The common ignore group is not added by default.')
		manGrp.add_argument('-n', '--name', action='store', nargs='?', default=None, help='The display name of the repository.')
		manGrp.add_argument('-d', '--description', action='store', nargs='?', default=None, help='The description of the repository.')
		manGrp.add_argument('-c', '--contact', action='store', nargs='?', default=None, help='The contact point of a repository.')
		manGrp.add_argument('-r', '--rename', action='store', nargs='?', default=None, help='The new storage-name for a repository.')

		# Examine the supplied arguments and perform requested actions.
		args = parser.parse_args()
		repo = __extract_values(args)

		if args.list:
			manager.list()
		elif args.plugin is not None:
			if args.enable:
				manager.plugInEnable(repo, args.plugin[0], args.script_path)
			elif args.disable:
				manager.plugInDisable(repo, args.plugin[0])
			elif args.setting is not None:
				if len(args.setting[1]) > 0:
					manager.setPlugInSetting(repo, args.plugin[0], args.setting[0], args.setting[1])
				else:
					manager.setPlugInSetting(repo, args.plugin[0], args.setting[0], None)
			else:
				print >>sys.stderr, "Failure determine requested plug-in '%s' option." % args.plugin[0]
				exit(1)
		elif args.action is None:
			print >>sys.stderr, "No repository operation argument supplied."
			exit(1)
		elif ('create' == args.action[0]) or ('c' == args.action[0]):
			manager.create(repo, args.ignore, args.skip_common)
		elif ('modify' == args.action[0]) or ('m' == args.action[0]):
			manager.modify(repo, args.storage, args.ignore, args.skip_common)
		elif ('delete' == args.action[0]) or ('d' == args.action[0]):
			manager.delete(repo)
		elif ('register' == args.action[0]) or ('r' == args.action[0]):
			manager.register(repo)
		else:
			print >>sys.stderr, "Unknown repository action supplied '%s'." % args.action[0]
			exit(1)
		exit(0)
	except (ValueError, RuntimeError) as error:
		print >>sys.stderr, "Execution Failed: %s" % error
		exit(2)
	except Exception as error:
		import traceback

		print >>sys.stderr, "Unexpected error: %s\n" % error, '-' * 60, '\n', traceback.format_exc(), '-' * 60
		exit(3)
