# -*- coding: utf-8 -*-
'''*****************************************************************************
***  This file is part of HgWeb Manager.                                     ***
***                                                                          ***
***  Copyright (C) 2014 - 2015                                               ***
***  CodeGNU Solutions <Licensing _AT_ CodeGNU _DOT_ com>                    ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''

from manager import settings


class Repository(object):
	""" Describes the properties of a repository managed by HWM and served up
	by HgWeb.
	"""

	# The directory name of the managed repository.
	__storageName = None
	# The displayed web-site name of the managed repository.
	__displayName = None
	# The displayed web-site description of the managed repository.
	__description = None
	# The displayed web-site contact information of the managed repository.
	__contact = None
	# The Mercurial repository interface.
	__repo = None
	# The collection of plug-ins enabled for a repository.
	__plugIns = None

	@property
	def StorageName(self):
		""" Gets the name of the directory where the managed repository is located. """
		return self.__storageName

	@property
	def DisplayName(self):
		""" Gets the name to display for the managed repository. """
		return self.__displayName

	@DisplayName.setter
	def DisplayName(self, value):
		""" Sets the name of the managed repository to display. """
		self.__displayName = value

	@property
	def Description(self):
		""" Gets the description of the managed repository. """
		return self.__description

	@Description.setter
	def Description(self, value):
		""" Sets the description of the managed repository to display. """
		self.__description = value

	@property
	def Contact(self):
		""" Gets the contact point for the managed repository. """
		return self.__contact

	@Contact.setter
	def Contact(self, value):
		""" Sets the contact point for the managed repository. """
		self.__contact = value

	@property
	def PlugIns(self):
		""" Gets the collection of enabled plug-ins for the managed repository. """
		return self.__plugIns

	def __init__(self, repoName=None):
		""" Initialises the object with the values from a managed repository

		@param[in] repoName		The storage name of the managed repository
								to load.
		"""
		from mercurial import ui, hg, error
		import os

		self.__storageName = repoName
		if repoName:
			try:
				self.__repo = hg.repository(ui.ui(), settings.RepositoryPath + os.sep + self.__storageName)
				self.__displayName = self.__repo.ui.config('web', 'name', default=None)
				self.__description = self.__repo.ui.config('web', 'description', default=None)
				self.__contact = self.__repo.ui.config('web', 'contact', default=None)
				plugInList = self.__repo.ui.configitems('extensions')
				self.__plugIns = []
				for plugIn in plugInList:
					self.__plugIns.append(RepositoryPlugin(plugIn[0], plugIn[1], self.__repo.ui.configitems(plugIn[0])))
			except error.RepoError:
				pass

	def __eq__(self, other):
		""" Determines if two repositories are the same repository.

		@param[in] other	The repository on the right-hand-side of the
							equality operator to compare against.

		@return Gives true when both repositories represent the same one,
		else-wise gives false.
		"""
		return self.__storageName == other.__storageName

	def save(self):
		""" Commits the repository detail properties to the repository
		configuration file.

		@throws IOError		When the configuration file for the repository
							fails to be written.
		"""
		import os

		with open(settings.RepositoryPath + os.sep + self.__storageName + os.sep + '.hg' + os.sep + 'hgrc', 'w') as hgrc:
			#{ Repository Information Section

			hgrc.write('[web]\n')
			if self.DisplayName is None:
				hgrc.write('name = ' + self.StorageName + '\n')
			else:
				hgrc.write('name = ' + self.DisplayName + '\n')

			if self.Description is not None:
				hgrc.write('description = ' + self.Description + '\n')

			if self.Contact is not None:
				hgrc.write('contact = ' + self.Contact + '\n')

			#} Repository Information Section

			#{ Plug-in Section

			if len(self.PlugIns) > 0:
				hgrc.write('\n[extensions]\n')
				for plugIn in self.PlugIns:
					if plugIn.scriptFile is None:
						hgrc.write(plugIn.name + ' =\n')
					else:
						hgrc.write(plugIn.name + ' = ' + plugIn.scriptFile + '\n')

			#} Plug-in Section

			#{ Plug-in Settings Section

			for plugIn in self.PlugIns:
				if len(plugIn.settings) > 0:
					hgrc.write("\n[%s]" % plugIn.name)
					for setting in plugIn.settings:
						hgrc.write("\n%s = %s" % (setting.name, setting.value))

			#} Plug-in Settings Section

		self = Repository(self.StorageName)  # Needed for when repositories are re-named.

	def __str__(self):
		""" Converts the object contents into a human-friendly string. """
		repo = "%s\n\tDisplay: %s\n\tDescription: %s\n\tContact: %s" % (self.StorageName, self.DisplayName, self.Description, self.Contact)
		if len(self.PlugIns) > 0:
			repo = repo + "\n\tPlug-ins:"
			for plugIn in self.PlugIns:
				repo = repo + "\n\t\t%s" % plugIn
		return repo

	def addPlugIn(self, plugIn):
		""" Makes a plug-in in a repository enabled.

		@param[in] plugIn	The plug-in that is to be enabled.

		@post The object now has the supplied plug-in as being enabled.

		@throws ValueError		When the supplied plug-in is already enabled.
		"""
		if plugIn not in self.PlugIns:
			self.__plugIns.append(plugIn)
		else:
			raise ValueError("The plug-in '%s' is already enabled." % plugIn.name)

	def removePlugIn(self, named):
		""" Makes a plug-in in a repository disabled.

		@param[in] named	The name of the plug-in that is to be disabled.

		@post The object no longer has the supplied plug-in as being enabled.

		@throws ValueError		When the supplied plug-in is already disabled.
		"""
		for plugIn in self.PlugIns:
			if named == plugIn.name:
				self.__plugIns.remove(plugIn)
				return
		raise ValueError("The plug-in '%s' is already disabled." % named)

	def setSetting(self, plugInName, name, value):
		""" Set's the setting of a plug-in
		@details When the supplied value is None, the setting is removed.

		@param[in] plugInName		The name of the plug-in whose setting is
									being set.
		@param[in] name				The name of the setting being set.
		@param[in] value			The value for the setting.

		@post The setting for the plug-in is modified to the supplied value.

		@throws ValueError	When the supplied plug-in name is not enabled.
		"""
		for plugIn in self.PlugIns:
			if plugInName == plugIn.name:
				plugInIndex = self.PlugIns.index(plugIn)
				self.__plugIns[plugInIndex].setPlugInSetting(name, value)
				return
		raise ValueError("The plug-in '%s' is not enabled." % plugInName)


class RepositoryPlugin(object):
	""" Manages a plug-in for a repository. """
	class PluginSetting(object):
		""" Manages a plug-in setting found inside a repository. """

		# The name of the plug-in setting.
		__name = None
		# The setting value for the of the plug-in option.
		__value = None

		@property
		def name(self):
			""" Accessor to the option name for a plug-in. """
			return self.__name

		@property
		def value(self):
			""" Accessor to the option value for a plug-in option. """
			return self.__value

		@value.setter
		def value(self, value):
			""" Mutator to the option value for a plug-in option. """
			self.__value = value

		def __init__(self, name, value=None):
			""" Initialises a plug-in setting.

			@param[in] name		The name of the plug-in setting option.
			@param[in] name		The value for the plug-in setting option.

			@post The object is ready for use.
			"""
			self.__name = name
			self.__value = value

		def __str__(self):
			""" Constructs a string representation of the object and it's values.

			@return The representation in the form name => value
			"""
			return "%s => %s" % (self.name, self.value)
	# The name of a repository plug-in.
	__name = None
	# The path and file-name of the script for the repository plug-in.
	__scriptFile = None
	# The supplied option settings for the repository plug-in.
	__settings = None

	@property
	def name(self):
		""" The accessor to the name of a repository plug-in. """
		return self.__name

	@property
	def scriptFile(self):
		""" The accessor to the path and file-name of the script for the
		repository plug-in.
		"""
		return self.__scriptFile

	@property
	def settings(self):
		""" The accessor to the supplied option settings for the repository
		plug-in.
		"""
		return self.__settings

	def __init__(self, name, scriptFile=None, plugInSettings=[]):
		""" Initialises a plug-in.
		@details The supplied plug-in setting must be a key-value pair
		co-responding to the plug-in option name and it's value respectively.

		@param[in] name				The name of the plug-in.
		@param[in] scriptFile		The path and file name of the script which
									runs the plug-in.
		@param[in] plugInSettings	A list of settings for the plug-in.

		@post The object is ready for use.
		"""
		import os

		self.__name = name
		if (scriptFile is not None and len(scriptFile) > 0):
			if os.path.exists(scriptFile):
				self.__scriptFile = scriptFile
			else:
				raise ValueError("The plug-in file '%s' was not found." % scriptFile)

		self.__settings = []
		for plugInSetting in plugInSettings:
			self.__settings.append(RepositoryPlugin.PluginSetting(plugInSetting[0], plugInSetting[1]))

	def __eq__(self, other):
		""" Determines if two repository plug-ins are the same.

		@param[in] other	The plug-in on the right-hand-side of the
							equality operator to compare against.

		@return Gives true when both plug-ins represent the same one,
		else-wise gives false.
		"""
		return self.name == other.name

	def __str__(self):
		""" Converts the object contents into a human-friendly string. """
		plugIn = ""
		if self.scriptFile is None:
			plugIn = plugIn + self.name
		else:
			plugIn = plugIn + "%s -> %s" % (self.name, self.scriptFile)
		for setting in self.settings:
			plugIn = plugIn + "\n\t%s" % setting
		return plugIn

	def setPlugInSetting(self, name, value):
		""" Set's the plug-in setting.
		@details When the plug-in setting already exists the supplied value
		overwrites the existing one. When the setting does not exist, a new
		entry is created. When the supplied value is None, the setting is
		removed from the plug-in.

		@param[in] name				The name of the setting being set.
		@param[in] value			The value for the setting.

		@post The setting for the plug-in is modified to the supplied value.
		"""
		for setting in self.settings:
			if name == setting.name:
				if value is None:
					self.__settings.remove(setting)
				else:
					settingIndex = self.settings.index(setting)
					self.__settings[settingIndex].value = value
				return
		self.__settings.append(RepositoryPlugin.PluginSetting(name, value))


class ManagedCollection(object):
	""" A container of all the managed repositories. """

	# All the managed repositories
	__repositories = []

	@property
	def Repositories(self):
		"""A collection of all managed repositories.

		@return An array of the managed repositories.
		"""
		return self.__repositories

	def __init__(self):
		""" Initialises the container with all the managed repositories.

		@throws RuntimeError	When the HgWeb configuration cannot be read.
		"""
		from ConfigParser import SafeConfigParser
		import os

		parser = SafeConfigParser()
		if not parser.read(settings.HgWebPath + os.sep + 'hgweb.config'):
			raise RuntimeError("Failed to read HgWeb configuration.")

		for name, path in parser.items('paths'):
			self.__repositories.append(Repository(name))

	def __str__(self):
		""" Converts the object contents into a human-friendly string. """
		collection = ""
		for repository in self.Repositories:
			collection = collection + str(repository) + '\n'
		collection.rstrip('\n')
		return collection
